package com.uas.message.controller;

import com.uas.message.common.domain.Page;
import com.uas.message.common.domain.SimpleMessage;
import com.uas.message.sms.domain.SmsLog;
import com.uas.message.sms.domain.SmsReceiver;
import com.uas.message.sms.domain.SmsSender;
import com.uas.message.sms.domain.SmsTemplate;
import com.uas.message.sms.service.SmsReceiverService;
import com.uas.message.sms.service.SmsSenderService;
import com.uas.message.sms.service.SmsService;
import com.uas.message.sms.service.SmsTemplateService;
import com.uas.message.viewmodel.SimpleReceiver;
import com.uas.message.viewmodel.SimpleSender;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.List;
import java.util.Map;

/**
 * 处理web控制台的短信及短信模板等的操作
 * 
 * @author yingp
 *
 */
@Controller
@RequestMapping("/sms")
public class SmsController {

	@Autowired
	private SmsService smsService;

	@Autowired
	private SmsSenderService smsSenderService;

	@Autowired
	private SmsReceiverService smsReceiverService;

	@Autowired
	private SmsTemplateService smsTemplateService;

	@RequestMapping
	public ModelAndView getHomePage() {
		ModelMap map = new ModelMap();
		map.put("templates", smsTemplateService.findTopUsed(5));
		map.put("senders", smsSenderService.findTopUsed(5));
		map.put("receivers", smsReceiverService.findTopUsed(5));
		return new ModelAndView("sms", map);
	}

	/**
	 * 短信模板列表
	 * 
	 * @return
	 */
	@RequestMapping(value = "/template", method = RequestMethod.GET)
	public ModelAndView getTemplateListPage() {
		ModelMap map = new ModelMap();
		map.put("templates", smsTemplateService.findAll());
		// TODO
		map.put("userName", "xxx");
		return new ModelAndView("sms_tpl", map);
	}

	/**
	 * 新增、修改模板
	 * 
	 * @param mailSender
	 * @return
	 */
	@RequestMapping(value = "/template", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap saveTemplate(SmsTemplate smsTemplate) {
		smsTemplate = smsTemplateService.save(smsTemplate);
		return new ModelMap("templateId", smsTemplate.getId());
	}

	/**
	 * 查找短信模板
	 * 
	 * @param templateId
	 * @return
	 */
	@RequestMapping(value = "/template/{templateId}", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getTemplate(@PathVariable("templateId") String templateId) {
		return new ModelMap("template", smsTemplateService.findOne(templateId));
	}

	/**
	 * 删除短信模板
	 * 
	 * @param templateId
	 */
	@RequestMapping(value = "/template/{templateId}", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteTemplate(@PathVariable("templateId") String templateId) {
		smsTemplateService.delete(templateId);
	}

	/**
	 * 新增、修改发送方资料
	 * 
	 * @param mailSender
	 * @return
	 */
	@RequestMapping(value = "/sender", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap saveSender(SmsSender smsSender) {
		smsSender = smsSenderService.save(smsSender);
		return new ModelMap("senderId", smsSender.getId());
	}

	/**
	 * 发送者列表
	 * 
	 * @return
	 */
	@RequestMapping(value = "/sender", method = RequestMethod.GET)
	public ModelAndView getSenderListPage() {
		ModelMap map = new ModelMap();
		map.put("senders", smsSenderService.findAll());
		// TODO
		map.put("userName", "xxx");
		return new ModelAndView("sms_sender", map);
	}

	/**
	 * 查找全部发送者
	 * 
	 * @return
	 */
	@RequestMapping(value = "/sender/simpleList", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getSenders() {
		return new ModelMap("senders", SimpleSender.from(smsSenderService.findAll()));
	}

	/**
	 * 查找发送者
	 * 
	 * @param senderId
	 * @return
	 */
	@RequestMapping(value = "/sender/{senderId}", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getSenderPage(@PathVariable("senderId") String senderId) {
		return new ModelMap("sender", smsSenderService.findOne(senderId));
	}

	/**
	 * 删除发送方资料
	 * 
	 * @param senderId
	 */
	@RequestMapping(value = "/sender/{senderId}", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteSender(@PathVariable("senderId") String senderId) {
		smsSenderService.delete(senderId);
	}

	/**
	 * 新增、修改接收方资料
	 * 
	 * @param mailSender
	 * @return
	 */
	@RequestMapping(value = "/receiver", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap saveReceiver(SmsReceiver smsReceiver) {
		smsReceiver = smsReceiverService.save(smsReceiver);
		return new ModelMap("receiverId", smsReceiver.getId());
	}

	/**
	 * 接收者列表
	 * 
	 * @return
	 */
	@RequestMapping(value = "/receiver", method = RequestMethod.GET)
	public ModelAndView getReceiverListPage() {
		ModelMap map = new ModelMap();
		map.put("receivers", smsReceiverService.findAll());
		// TODO
		map.put("userName", "xxx");
		return new ModelAndView("sms_receiver", map);
	}

	/**
	 * 查找全部接收者
	 * 
	 * @return
	 */
	@RequestMapping(value = "/receiver/simpleList", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getReceivers() {
		return new ModelMap("receivers", SimpleReceiver.from(smsReceiverService.findAll()));
	}

	/**
	 * 查找接收者
	 * 
	 * @param senderId
	 * @return
	 */
	@RequestMapping(value = "/receiver/{receiverId}", method = RequestMethod.GET)
	@ResponseBody
	public ModelMap getReceiverPage(@PathVariable("receiverId") String receiverId) {
		return new ModelMap("receiver", smsReceiverService.findOne(receiverId));
	}

	/**
	 * 删除接收方资料
	 * 
	 * @param senderId
	 */
	@RequestMapping(value = "/receiver/{receiverId}", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteReceiver(@PathVariable("receiverId") String receiverId) {
		smsReceiverService.delete(receiverId);
	}

	/**
	 * 发送短信 oneToOne
	 * 
	 * @param message
	 * @return
	 */
	@RequestMapping(value = "/send", method = RequestMethod.POST)
	@ResponseBody
	public SmsLog sendByTemplate(@RequestBody SimpleMessage message) {
		return smsService.send(message);
	}

	/**
	 * 发送短信 oneToMany
	 * 
	 * <pre>
	 * 无模板，使用默认发送方来发送
	 * </pre>
	 * 
	 * @param tos
	 *            接收地址
	 * @param content
	 *            内容
	 * @return
	 */
	@RequestMapping(value = "/send/default", method = RequestMethod.POST)
	@ResponseBody
	public List<Map<String, Object>> sendByDefault(@RequestParam(required = true) String tos, @RequestParam(required = true) String content) {
		return smsService.sendByDefault(tos.split(","), content);
	}

	/**
	 * 发送短信 oneToMany
	 * 
	 * @param message
	 * @return
	 */
	@RequestMapping(value = "/send/o2m", method = RequestMethod.POST)
	@ResponseBody
	public List<SmsLog> sendAllByTemplate(@RequestBody SimpleMessage message) {
		return smsService.sendAll(message);
	}

	/**
	 * 发送短信 manyToMany
	 * 
	 * @param templateId
	 * @param receiver
	 * @param params
	 * @return
	 */
	@RequestMapping(value = "/send/m2m", method = RequestMethod.POST)
	@ResponseBody
	public List<SmsLog> sendAllByTemplate(@RequestBody List<SimpleMessage> messages) {
		return smsService.sendAll(messages);
	}

	/**
	 * 查看发送日志
	 * 
	 * @param templateId
	 * @param success
	 * @param pageNumber
	 * @param pageSize
	 * @return
	 */
	@RequestMapping(value = "/send/log", method = RequestMethod.GET)
	@ResponseBody
	public Page<SmsLog> getLogs(String templateId, String receiver, Boolean success, int pageNumber, int pageSize) {
		return smsService.findLogs(templateId, receiver, success, pageNumber, pageSize);
	}

	/**
	 * 清除指定模板的发送日志
	 * 
	 * @param templateId
	 * @param pageNumber
	 * @param pageSize
	 * @return
	 */
	@RequestMapping(value = "/send/log", method = RequestMethod.DELETE)
	@ResponseBody
	public void deleteLogs(String templateId) {
		smsService.deleteLogs(templateId);
	}

}
