var app = {
	loading: function(is) {
		$('#loading').css('display', is ? 'block': 'none');
	},
	info: function(message, title) {
		toastr.info(message, title, {positionClass: 'toast-top-center', timeOut: 1000});
	},
	error: function(message, title) {
		toastr.error(message, title, {positionClass: 'toast-top-center'});
	},
	//============CRUD about sender===============
	editSender : function(event, senderId) {
		senderId = event ? $(event.target).data("id") : senderId;
		if (senderId) {
			$.getJSON('/mail/sender/' + senderId, function(data){
				app.showSenderModal(data.sender);
			});
		} else {
			// new sender
			app.showSenderModal({
				port: 25
			});
		}
	},
	getSenders: function(callback) {
		$.getJSON('/mail/sender/simpleList', function(data){
			callback.call(null, data.senders);
		});
	},
	showSenderModal: function(sender) {
		$('#senderModal').showModal(function(){
			var scope = $(this);
			scope.renderData(sender);
			scope.find('.btn-submit').click(scope.find('form'), app.saveSender);
			if(sender.id) {
				scope.find('.btn-del').click(sender, app.deleteSender);
				scope.find('.btn-del').show();
			} else {
				scope.find('.btn-del').hide();
			}
		});
	},
	saveSender : function(event) {
		var form = event.data, sender = form.serializeObject();
		app.loading(true);
		$.post('/mail/sender', sender, function(data){
			app.loading(false);
			if(data.senderId) {
				window.location.reload();
			} else if (data.errMsg) {
				app.error(data.errMsg);
			}
		}, 'json');
	},
	deleteSender: function(event){
		var senderId = event.data.id;
		if(senderId) {
			app.loading(true);
			$.del('/mail/sender/' + senderId, function(data) {
				app.loading(false);
				if (data && data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			});
		}
	},
	//============CRUD about receiver===============
	editReceiver : function(event, receiverId) {
		receiverId = event ? $(event.target).data("id") : receiverId;
		if (receiverId) {
			$.getJSON('/mail/receiver/' + receiverId, function(data){
				app.showReceiverModal(data.receiver);
			});
		} else {
			// new receiver
			app.showReceiverModal({
				userType: 'DEFAULT'
			});
		}
	},
	getReceivers: function(callback) {
		$.getJSON('/mail/receiver/simpleList', function(data){
			callback.call(null, data.receivers);
		});
	},
	showReceiverModal: function(receiver) {
		$('#receiverModal').showModal(function(){
			var scope = $(this);
			scope.renderData(receiver);
			scope.find('.btn-submit').click(scope.find('form'), app.saveReceiver);
			if(receiver.id) {
				scope.find('.btn-del').click(receiver, app.deleteReceiver);
				scope.find('.btn-del').show();
			} else {
				scope.find('.btn-del').hide();
			}
			var toggleUserType = function(userType) {
				if('DEFAULT' == userType) {
					scope.find('#receiverModal_user_sql').hide();
					scope.find('#receiverModal_user_default').show();
				} else {
					scope.find('#receiverModal_user_default').hide();
					scope.find('#receiverModal_user_sql').show();
				}
			};
			toggleUserType(receiver.userType);
			scope.find(':radio[name=userType]').change(function(){
				toggleUserType($(this).val());
			});
		});
	},
	saveReceiver : function(event) {
		var form = event.data, receiver = form.serializeObject();
		app.loading(true);
		$.post('/mail/receiver', receiver, function(data){
			app.loading(false);
			if(data.receiverId) {
				window.location.reload();
			} else if (data.errMsg) {
				app.error(data.errMsg);
			}
		}, 'json');
	},
	deleteReceiver: function(event){
		var receiverId = event.data.id;
		if(receiverId) {
			app.loading(true);
			$.del('/mail/receiver/' + receiverId, function(data) {
				app.loading(false);
				if (data && data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			});
		}
	},
	//============CRUD about template===============
	editTemplate : function(event, templateId) {
		templateId = event ? $(event.target).data("id") : templateId;
		if (templateId) {
			$.getJSON('/mail/template/' + templateId, function(data){
				app.showTemplateModal(data.template);
			});
		} else {
			// new template
			app.showTemplateModal({
				content: '<div>\n\thello!\n</div>'
			});
		}
	},
	resizeEditor: function(pageScope, editor){
		var bodyInnerHeight = pageScope.find('.modal-body').height(),
			editorHeight = pageScope.find('.CodeMirror').height(),
			formHeight = pageScope.find('form').outerHeight();
		editor.setSize(null, bodyInnerHeight - formHeight + editorHeight);
	},
	showTemplateModal: function(template) {
		$('#templateModal').showModal({
			show: function() {
				var scope = $(this), editTab = scope.find('#templateModal_edit');
				editTab.renderData(template);
				if(template.id) {
					editTab.find('.btn-del').click(template, app.deleteTemplate);
					editTab.find('.btn-del').show();
					if (template.attachs) {
						editTab.find('#attachs').appendTpl('<div class="x-attach-wrap"><b class="x-attach-file"><i class="fa fa-file-image-o"></i></b><div class="x-attach-info"><span>#{name}</span><div class="x-attach-toolbar-bg"></div><div class="x-attach-toolbar"><a href="#{path}" target="_blank" title="下载"><i class="fa fa-download fa-fw"></i></a><a class="x-btn-attach-del" data-path="#{path}" title="删除"><i class="fa fa-times fa-fw"></i></a></div></div></div>', template.attachs);
						editTab.find('.x-btn-attach-del').click(template, app.deleteAttach);
					}
				} else {
					editTab.find('.btn-del').hide();
					scope.find('a[data-toggle="tab"]').hide();
				}
			},
			shown: function(){
				var scope = $(this), editTab = scope.find('#templateModal_edit');
				// default is edit tab
				var editor = CodeMirror.fromTextArea(editTab.find('textarea')[0], {
					lineNumbers: true
				});
				// resize
				app.resizeEditor(editTab, editor);
				editTab.find('.btn-submit').click({
					element: editTab.find('form'),
					resolve: function() {
						return {
							content: editor.getValue()
						};
					}
				}, app.saveTemplate);
				editTab.find('#attach').change({
					element: editTab.find('form'),
					resolve: function() {
						return {
							content: editor.getValue()
						};
					}
				}, app.saveTemplate);
				// load senders
				app.getSenders(function(senders){
					var box = editTab.find('#senders-box'), curr_id = template.senderId;
					if(curr_id) {
						for(var i in senders) {
							if(senders[i].id == curr_id) {
								box.find('.active').text(senders[i].description);
								break;
							}
						}
					}
					box.find('ul').appendTpl('<li><a href="#" title="#{username}" data-id="#{id}">#{description}</a></li>', senders);
					box.find('a').click(function() {
						var that = $(this);
						box.find('.active').text(that.text());
						editTab.find('input[name=senderId]').val(that.data('id'));
					});
				});
				// listen on tab active
				scope.find('a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
					var tabId = $(e.target).attr('aria-controls'), tab = scope.find("#" + tabId),
						ready = tab.attr('viewready');
					if(!ready) {
						tab.attr('viewready', true);
						if(tabId == 'templateModal_log') {
							app.showSendLog({
								templateId: template.id,
								pageNumber: 1,
								pageSize: 10
							});
							tab.find('.btn-clearlog').click(template, app.deleteSendLog);
						} else if(tabId == 'templateModal_send') {
							app.getReceivers(function(receivers){
								var box = tab.find("#receivers-box");
								box.appendTpl('<label class="checkbox-inline"><input type="checkbox" name="receiverIds" value="#{id}"> #{description}</label>', receivers);
							});
							tab.find('textarea[name=params]').val('{\n  "param1": "hello",\n  "param2": "world"\n}');
							var editor = CodeMirror.fromTextArea(tab.find('textarea[name=params]')[0], {
								lineNumbers: true,
								mode: 'application/json'
							});
							// resize
//							app.resizeEditor(tab, editor);
							tab.find('.btn-send').click({
								element: tab.find('form'),
								resolve: function() {
									return {
										templateId: template.id,
										params: $.parseJSON(editor.getValue())
									};
								}
							}, app.sendMail);
						}
					}
				});
			}
		});
	},
	saveTemplate : function(event) {
		var form = event.data.element, template = $.extend(form.serializeObject(), event.data.resolve.call());
		app.loading(true);
		$.ajaxFileUpload({
			url: '/mail/template',
			secureuri: false,
			fileElementId: 'attach',
			data: template,
			dataType: 'json',
			success: function (data){
				app.loading(false);
				if(data.templateId) {
					window.location.hash = '/template/' + data.templateId;
					window.location.reload();
				} else if (data.errMsg) {
					app.error(data.errMsg);
				}
			},
			error: function (){
				app.loading(false);
				app.error('出现错误');
			}
		});
	},
	deleteTemplate: function(event){
		var templateId = event.data.id;
		if(templateId) {
			app.loading(true);
			$.del('/mail/template/' + templateId, function(data) {
				app.loading(false);
				if (data && data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			});
		}
	},
	deleteAttach: function(event) {
		var templateId = event.data.id, path = $(this).data('path');
		if (templateId && path) {
			app.loading(true);
			$.del('/mail/template/attach', {
				templateId: templateId,
				path: path
			}, function(data) {
				app.loading(false);
				if (data && data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.hash = '/template/' + templateId;
					window.location.reload();
				}
			});
		}
	},
	sendMail: function(event) {
		var form = event.data.element, mail = $.extend(form.serializeObject(), event.data.resolve.call());
		mail.receivers = (mail.receivers ? (mail.receivers.split(/\n+|\t+|\s+|,|;/)) : null);
		app.loading(true);
		$.ajax({
			url: '/mail/send/o2m',
			type: 'POST',
			contentType:'application/json; charset=UTF-8',
			data: JSON.stringify(mail),
			processData: false,
			dataType: 'json',
			success: function(data){
				app.loading(false);
				if (data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			},
			error: function() {
				window.location.reload();
			}
		});
	},
	//============about log===============
	showSendLog: function(params) {
		$('.pagination').pagination({
			url: '/mail/send/log',
			params: params,
			renderData: function(content){
				var list = $('.x-log-list .x-common-list');
				if(content.length == 0) {
					list.empty();
					$('.x-log-list .x-empty').show();
				} else {
					$('.x-log-list .x-empty').hide();
					$.each(content, function(i, d){
						d.sendDate = new Date(d.sendDate).format('yy-mm-dd h:i');
					});
					list.appendTpl('<div class="x-item"><div class="x-icon"><i class="fa fa-envelope-o"></i></div><p><span class="x-title text-info">#{description}</span><span class="pull-right x-text-muted">#{sendDate}</span></p><div class="x-text-muted">#{sender} <i class="fa fa-fw fa-mail-forward"></i> #{receiver}#{fn:showStatus}</div></div>', content, {
						showStatus: function(data) {
							if (data.success) {
								return '<span class="pull-right" title="发送成功"><i class="fa fa-check"></i></span>';
							} else if (data.errMsg) {
								return '<span class="pull-right" data-toggle="popover" data-trigger="hover" data-placement="left" data-content="' + data.errMsg + '"><i class="fa fa-times"></i></span>';
							}
						}
					});
					$('[data-toggle="popover"]').popover();
				}
			}
		});
	},
	searchSendLog: function(event) {
		var target = $(event.target), form = $('.x-nav-tabs form'), data = form.serializeObject();
		if (!target.is('a')) {
			target = $('.x-nav-tabs .active a');
		}
		data.success = target.data('success');
		data.pageNumber = 1;
		app.showSendLog(data);
		$('.x-nav-tabs .active').removeClass('active');
		target.parent().addClass('active');
	},
	deleteSendLog: function(event) {
		var templateId = event.data.id;
		if(templateId) {
			app.loading(true);
			$.del('/mail/send/log', {
				templateId: templateId
			}, function(data) {
				app.loading(false);
				if (data && data.errMsg) {
					app.error(data.errMsg);
				} else {
					window.location.reload();
				}
			});
		}
	},
	//============document ready===============
	init : function() {
		$('.x-sender-list .x-editor-trigger').click(app.editSender);
		$('.x-receiver-list .x-editor-trigger').click(app.editReceiver);
		$('.x-tpl-list .x-editor-trigger').click(app.editTemplate);
		new Router({
			'#/home': function() {
				app.showSendLog();
				$('.x-nav-tabs a').click(app.searchSendLog);
				$('.x-nav-tabs .btn-search').click(app.searchSendLog);
			},
			'#/template/(.*)': function(id) {
				app.editTemplate(null, id);
			},
			'#/sender/(.*)': function(id) {
				app.editSender(null, id);
			},
			'#/receiver/(.*)': function(id) {
				app.editReceiver(null, id);
			}
		}).start();
		app.loading(false);
	}
};
$(document).ready(function() {
	app.init();
});